@@ -1,19 +1,20 @@
 import re
-from enum import Enum
-from functools import lru_cache
-from typing import Optional, Type
 from abc import ABC, abstractclassmethod
+from enum import Enum
+from functools import lru_cache, partial
+from typing import Optional, Tuple
 
+import docx.document
 from docx import Document
 from docx.enum.text import WD_COLOR_INDEX
 from docx.shared import Inches, Pt
 from docx.table import _Cell, _Column
 from docx.text.paragraph import Paragraph
-import docx.document
 
-from .classes import QuestionType, Subject, TossUpBonus, ErrorLogger
+from .classes import ErrorLogger, QuestionType, Subject, TossUpBonus
 from .docx_utils import (
     clear_cell,
+    column_indexer,
     copy_run_formatting,
     fuse_consecutive_runs,
     highlight_cell_text,
@@ -39,6 +40,22 @@ COL_WIDTHS = (
     1.44,
 )
 
+COL_MAPPING = {
+    "TUB": 0,
+    "Subj": 1,
+    "Ques": 2,
+    "LOD": 3,
+    "LOD-A": 4,
+    "Set": 5,
+    "Rd": 6,
+    "Q Letter": 7,
+    "Author": 8,
+    "ID": 9,
+    "Source": 10,
+    "Comments": 11,
+    "Subcat": 12,
+}
+
 
 class QuestionFormatterState(Enum):
     Q_START = 0
@@ -52,11 +69,11 @@ class CellFormatter(ABC):
 
     @abstractclassmethod
     def __init__(self, cell: _Cell, error_logger: Optional[ErrorLogger] = None):
-        pass
+        """All CellFormatters take the same input arguments."""
 
     @abstractclassmethod
     def format(self):
-        pass
+        """All CellFormatters have a format function."""
 
 
 def make_jans_shadings(table):
@@ -114,34 +131,35 @@ def initialize_table(
     table.autofit = False
     table.allow_autofit = False
 
-    table.cell(0, 0).paragraphs[0].add_run("TUB")
-    table.cell(0, 1).paragraphs[0].add_run("Subj")
-    ques_run = table.cell(0, 2).paragraphs[0].add_run("Ques")
+    _cells = table._cells
+    _col_count = table._column_count
+
+    col_iter = partial(
+        column_indexer, total_cells=len(_cells), col_count=_col_count, skip_header=True
+    )
+
+    for col_name, col_idx in COL_MAPPING.items():
+        # add header
+        table.cell(0, col_idx).paragraphs[0].add_run(col_name)
+        table.cell(0, col_idx).width = Inches(COL_WIDTHS[col_idx])
+
+        for cell_idx in col_iter(col_idx):
+            cell = _cells[cell_idx]
+            cell.width = Inches(COL_WIDTHS[col_idx])
+
+            if col_name == "Subj" and subj is not None:
+                cell.paragraphs[0].text = subj
+            elif col_name == "Set" and set is not None:
+                cell.paragraphs[0].text = set
+            elif col_name == "Author" and name is not None:
+                cell.paragraphs[0].text = name
+
+    # ques header is italicized
+    ques_run = table.cell(0, COL_MAPPING["Ques"]).paragraphs[0].runs[0]
     ques_run.italic = True
-    table.cell(0, 3).paragraphs[0].add_run("LOD")
-    table.cell(0, 4).paragraphs[0].add_run("LOD-A")
-    table.cell(0, 5).paragraphs[0].add_run("Set")
-    table.cell(0, 6).paragraphs[0].add_run("Rd")
-    table.cell(0, 7).paragraphs[0].add_run("Q Letter")
-    table.cell(0, 8).paragraphs[0].add_run("Author")
-    table.cell(0, 9).paragraphs[0].add_run("ID")
-    table.cell(0, 10).paragraphs[0].add_run("Source")
-    table.cell(0, 11).paragraphs[0].add_run("Comments")
-    table.cell(0, 12).paragraphs[0].add_run("Subcat")
 
     make_jans_shadings(table)
 
-    for idx, column in enumerate(table.columns):
-        for cell_idx, cell in enumerate(column.cells):
-            cell.width = Inches(COL_WIDTHS[idx])
-            if cell_idx > 0:
-                if idx == 1 and subj is not None:
-                    cell.paragraphs[0].text = subj
-                elif idx == 5 and set is not None:
-                    cell.paragraphs[0].text = set
-                elif idx == 8 and name is not None:
-                    cell.paragraphs[0].text = name
-
     if path is not None:
         document.save(path)
 
@@ -182,10 +200,9 @@ class QuestionCellFormatter(CellFormatter):
 
             if run_match:
                 self.q_type = QuestionType.from_string(run_match.group(1))
-                run_length = len(q_type_run.text)
                 # if the run contains more than the question type, split
                 # the run into two
-                if run_match.span()[1] < run_length:
+                if run_match.span()[1] < len(q_type_run.text):
                     q_type_run, _ = split_run_at(para, q_type_run, run_match.span()[1])
 
                 q_type_run.text = self.q_type.value
@@ -207,7 +224,7 @@ class QuestionCellFormatter(CellFormatter):
             # doing it this way ensures that any other formatting in
             # the stem is preserved (superscripts, subscripts, etc.)
             stem_run = para.runs[1]
-            stem_run.text = "".join(["    ", stem_run.text.lstrip()])
+            stem_run.text = f"    {stem_run.text.lstrip()}"
 
             self.state = QuestionFormatterState.STEM_END
         else:
@@ -218,10 +235,6 @@ class QuestionCellFormatter(CellFormatter):
         choice_match = self._choices_re.match(para.text)
 
         if choice_match:
-            # insert a blank paragraph before this one. only
-            # do this if we are looking for W), the first choice
-            if self.current_choice == 0:
-                para.insert_paragraph_before("")
 
             # the first run should contain the choice
             choice_run = para.runs[0]
@@ -258,10 +271,6 @@ class QuestionCellFormatter(CellFormatter):
         if answer_match:
             para.insert_paragraph_before("")
 
-            # set the font to all-caps for every run in the answer line
-            for run in para.runs:
-                run.text = run.text.upper()
-
             # for MC questions, additional checks to make sure
             # answer line matches choice
             if self.q_type is QuestionType.MULTIPLE_CHOICE:
@@ -288,7 +297,10 @@ class QuestionCellFormatter(CellFormatter):
                     # if it doesn't raise a linting error.
                     else:
                         choice_num = self._choices.index(test_choice_match.group(1))
-                        if self.choices_para[choice_num].text.upper() != answer_text:
+                        if (
+                            self.choices_para[choice_num].text.upper()
+                            != answer_text.upper()
+                        ):
                             highlight_paragraph_text(para, WD_COLOR_INDEX.YELLOW)
                             if self.error_logger is not None:
                                 self.error_logger.log_error(
@@ -424,9 +436,27 @@ class SubjectCellFormatter(CellFormatter):
         return self.cell
 
 
+class DifficultyFormatter(CellFormatter):
+    def __init__(self, cell: _Cell, error_logger: Optional[ErrorLogger] = None):
+        self.cell = cell
+        self.error_logger = error_logger
+
+    def format(self):
+
+        if self.cell.text:
+            try:
+                int(self.cell.text)
+            except ValueError:
+                highlight_cell_text(self.cell, WD_COLOR_INDEX.RED)
+                if self.error_logger is not None:
+                    self.error_logger.log_error("LOD should be blank or an integer.")
+
+        return self.cell
+
+
 def format_column(
     nsb_table_column: _Column,
-    formatter: Type[CellFormatter],
+    formatter: Optional[CellFormatter],
     error_logger: Optional[ErrorLogger] = None,
 ) -> _Column:
     """Utility function that applies a formatter to every cell in a column.
@@ -441,19 +471,31 @@ def format_column(
     -------
     _Column
     """
-    for idx, cell in enumerate(nsb_table_column.cells[1:]):
+    for idx, cell in enumerate(nsb_table_column):
         cell = preprocess_cell(cell)
         if error_logger is not None:
             error_logger.set_row(idx + 1)
-        cell_formatter = formatter(cell, error_logger)
-        if cell.text.strip() != "":
-            cell_formatter.format()
+        if cell.text.strip() != "" and formatter:
+            formatter(cell, error_logger).format()
     return nsb_table_column
 
 
-def format_table(table_doc: docx.document.Document, verbosity=True):
+def format_table(
+    table_doc: docx.document.Document, cols_to_format: Tuple[str], verbosity=True
+):
     """Formats a Word document containing a Science Bowl question table.
 
+    Specifically, this function makes sure the columns fit the following
+    criteria:
+
+    TUB: Contains only TOSS-UP or BONUS
+    Subj: Contains only one of the valid subject areas
+    Ques: Contains a properly-formatted Short Answer or Multiple Choice question
+    LOD: Contains only an integer or is blank.
+    Set: Contains no extra whitespace
+    Author: Contains no extra whitespace
+    Subcat: Contains no extra whitespace
+
     Parameters
     ----------
     table_doc : Document
@@ -463,17 +505,23 @@ def format_table(table_doc: docx.document.Document, verbosity=True):
         "TUB": TuBCellFormatter,
         "Subj": SubjectCellFormatter,
         "Ques": QuestionCellFormatter,
+        "LOD": DifficultyFormatter,
     }
     error_logger = ErrorLogger(verbosity)
 
-    columns = {}
+    _cells = table_doc.tables[0]._cells
+    _col_count = table_doc.tables[0]._column_count
 
-    columns["TUB"] = table_doc.tables[0].columns[0]
-    columns["Subj"] = table_doc.tables[0].columns[1]
-    columns["Ques"] = table_doc.tables[0].columns[2]
+    col_iter = partial(
+        column_indexer, total_cells=len(_cells), col_count=_col_count, skip_header=True
+    )
 
-    for col_name in columns.keys():
-        format_column(columns[col_name], FORMATTERS[col_name], error_logger)
+    for col_name in cols_to_format:
+        format_column(
+            [_cells[i] for i in col_iter(COL_MAPPING[col_name])],
+            FORMATTERS.get(col_name, None),
+            error_logger,
+        )
 
     if len(error_logger.errors) == 0:
         print("Found no errors 😄")
